package graphpack.mathdisp;

import java.awt.*;
import java.text.*;

//***********************************************************/
//*                                                         */
/** Klasse zur grafischen Ausgabe mathematischer Funktionen */
//*                                                         */
//***********************************************************/

public class Mathdisp {

/*-----------------*/
/* Graphic Context */
/*-----------------*/

/** Referenz auf einen Graphic-Context. Methode: setGraphicContext( g)*/
private Graphics gc = null;

/** Referenz auf die Dimensionierung des Graphic Context. Methode: setSize( dim)*/
private Dimension size = null;

/*-----------------*/
/* Achsenteilungen */
/*-----------------*/

/** Teilung der X-Achse. Methode: setDefCut( cuts) */
private int xCut = 5;

/** Teilung der Y-Achse. Methode: setValCut( cuts) */
private int yCut = 7;

/*--------------------------------*/
/* Teilstrichlnge und Richtungen */
/*--------------------------------*/

/** Lnge der Teilstriche. Methode: setMarkLen( len) */
private int markLen = 5;

/*-------------------*/
/* Funktionsbereiche */
/*-------------------*/

/** unterer Definitionsbereich. Methode: setDefArea( min, max) */
private double defMin = 0.0;

/** oberer Definitionsbereich. Methode: setDefArea( min, max) */
private double defMax = 0.0;

/** unterer Wertebereich. Methode: setValArea( min, max) */
private double valMin = 0.0;

/** oberer Wertebereich. Methode: setValArea( min, max) */
private double valMax = 0.0;

/*-----------------------------------------------*/
/* Parameter der Pfeile an den Koordinatenachsen */
/*-----------------------------------------------*/

/** Lnge des Stiels ohne Spitze. Methode: setShaftLength( len) */
private int shaftLength = 10;

/** Lnge der Spitze. Methode: setPeakLength( len) */
private int peakLen = 5;

/** halbe Breite der Spitze. Methode: setPeakWidth( width) */
private int peakWidth = 3;

/*--------------------------*/
/* Bezeichnungen der Achsen */
/*--------------------------*/

/** Name des Definitionsbereichs. Methode: setDefName( name) */
private String defName = "x";

/** Name des Wertebereichs. Methode: setValName( name) */
private String valName = "f(" + defName + ")";

/*------------------------------------------------*/
/* numerische Formatierung der Achsenbeschriftung */
/*------------------------------------------------*/

/** Formatierungstext des Definitionsbereichs. Methode: setDefFormat( form) */
private String formDef = "#.#";

/** Formatierung des Definitionsbereichs. Methode: setDefFormat( form) */
private DecimalFormat nfDef = new DecimalFormat( formDef);

/** Formatierungstext des Wertebereichs. Methode: setValFormat( form) */
private String formVal = "#.#";

/** Formatierung des Definitionsbereichs. Methode: setValFormat( form) */
private DecimalFormat nfVal = new DecimalFormat( formVal);

/*--------------------------------------*/
/* Fonts der vorhandenen Beschriftungen */
/*--------------------------------------*/

/** Font der Werte im Definitionsbereich. Methode: setDefFormFont( name, style, size) */
private Font defFormFont = new Font( "SansSerif",             Font.ITALIC, 14);

/** Font der Werte im Wertebereich. Methode: setValFormFont( name, style, size) */
private Font valFormFont = new Font( "SansSerif",             Font.PLAIN, 14);

/** Font des Namens des Definitionsbereichs. Methode: setDefNameFont( name, style, size) */
private Font defNameFont = new Font( "SansSerif", Font.BOLD + Font.ITALIC, 14);

/** Font des Namens des Wertebereichs. Methode: setValNameFont( name, style, size) */
private Font valNameFont = new Font( "SansSerif", Font.BOLD + Font.ITALIC, 14);

/*---------------------------------*/
/* Farben des Darstellungsbereichs */
/*---------------------------------*/

/** Hintergrundfarbe des Darstellungsbereichs. Methode: setBackground( col) */
private Color background = Color.lightGray;

/** Zeichenfarbe im Darstellungsbereich. Methode: setPaintColor( col) */
private Color paintColor = Color.black;

/** Farbe der Bezeichnung des Definitionsbereichs. Methode: setDefNameColor( col) */
private Color defNameColor = paintColor;

/** Farbe der Beschriftung des Definitionsbereichs. Methode: setDefFormColor( col)  */
private Color defFormColor = paintColor;

/** Farbe der Bezeichnung des Wertebereichs. Methode: setValNameColor( col) */
private Color valNameColor = paintColor;

/** Farbe der Beschriftung des Wertebereichs. Methode: setValFormColor( col) */
private Color valFormColor = paintColor;

/*-------------------------------------------*/
/* Abstand zu den Grenzen des Grafikbereichs */
/*-------------------------------------------*/

/** linker Abstand des Darstellungsbereichs */
private int bLeft  = 0;

/** rechter Abstand des Darstellungsbereichs */
private int bRight = 0;

/** unterer Abstand des Darstellungsbereichs */
private int bLower = 0;

/** oberer Abstand des Darstellungsbereichs */
private int bUpper = 0;

/*----------------------*/
/* Form der Koordinaten */
/*----------------------*/

/** Form der Koordinaten */
private int xyForm;

/*--------------------------------------------------*/
/* signifikante Ausdehnungen der Achsenbeschriftung */
/*--------------------------------------------------*/

/** Namensweite des Definitionsbereichs */
private int defNameWidth;

/** Namenshhe des Wertebereichs */
private int valNameHeight;

/*---------------------*/
/* Skalierungsfaktoren */
/*---------------------*/

/** Skalierungsfaktor des Definitionsbereichs */
private double xScale;

/** Skalierungsfaktor des Wertebereichs */
private double yScale;

/*-----------------------------------------------*/
/* Abstnde der Achsenteilungen in realen Werten */
/*-----------------------------------------------*/

/** Wert eines Teilungsabstands im Definitionsbereich */
private double defCut;

/** Wert eines Teilungsabstands im Wertebereich */
private double valCut;

/*------------------------------------------*/
/* Richtung der Markierungen auf den Achsen */
/*------------------------------------------*/

/** Richtung der Markierungen im Definitionsbereich */
private int markDef = -1;

/** Richtung der Markierungen im Wertebereich */
private int markVal =  1;

/*--------------------------------------------*/
/* Pfeilrichtungen und Koordinatenkreuzformen */
/*--------------------------------------------*/

/** Pfeil nach rechts, Koordinaten nach rechts offen */
private static final int RIGHT = 1;

/** Pfeil nach links, Koordinaten nach links offen */
private static final int LEFT  = 2;

/** Pfeil nach oben, Koordinaten nach oben offen */
private static final int UP    = 4;

/** Pfeil nach unten, Koordinaten nach unten offen */
private static final int DOWN  = 8;


 //*===============================*/
 /** Ermitteln der Koordinatenform */
 //*===============================*/

 private int getForm() {
 int retval = 0;

  if ((defMax >  0) && (defMin >= 0)) retval = (retval | RIGHT);        // NUR rechts
  if ((defMax <= 0) && (defMin <  0)) retval = (retval | LEFT);         // NUR links
  if ((defMax >  0) && (defMin <  0)) retval = (retval | RIGHT | LEFT); // rechts UND links

  if ((valMax >  0) && (valMin >= 0)) retval = (retval | UP);           // NUR rauf
  if ((valMax <= 0) && (valMin <  0)) retval = (retval | DOWN);         // NUR runter
  if ((valMax >  0) && (valMin <  0)) retval = (retval | UP | DOWN);    // rauf UND runter

  return retval;
 }
 
 //*===================================*/
 /** Maximale Textausdehnung ermitteln */
 //*===================================*/

 private int maxTextWidth( double cut, DecimalFormat nfDef, Font font) {
 int width;
 int retval = 0;

 Font        actfont = gc.getFont();
 FontMetrics fm;
  gc.setFont( font);
  fm = gc.getFontMetrics();
  gc.setFont( actfont);

  for( int i=0; i<=yCut; i++) {
   width = fm.stringWidth( nfDef.format( valMin + cut * i) + " ");
   if (retval < width) retval = width;
  }
  return retval;
 }

 //*==========================================*/
 /** Textbreite bei bestimmtem Font ermitteln */
 //*==========================================*/

 private int getTextWidth( String txt, Font font) {
 Font        actfont = gc.getFont();
 FontMetrics fm;

  gc.setFont( font);
  fm = gc.getFontMetrics();
  gc.setFont( actfont);
  return fm.stringWidth( txt);
 }
 
 //*========================================*/
 /** Texthhe bei bestimmtem Font ermitteln */
 //*========================================*/

 private int getTextHeight( Font font) {
 Font        actfont = gc.getFont();
 FontMetrics fm;

  gc.setFont( font);
  fm = gc.getFontMetrics();
  gc.setFont( actfont);
  return fm.getHeight();
 }
 
 //*======================================================*/
 /** Initialisierung einiger, hufig benutzter, Variablen */
 //*======================================================*/

 private void varInit() {
 FontMetrics fm;
 
  bLeft  = 0;
  bRight = 0;
  bLower = 0;
  bUpper = 0;

  xyForm = getForm();               // Form der Koordinaten

  valCut = Math.abs( valMax - valMin) / (double)yCut;
  defCut = Math.abs( defMax - defMin) / (double)xCut;

  defNameWidth  = getTextWidth( defName + " ", defNameFont);
  valNameHeight = getTextHeight( valNameFont);
 }
 
 //*==============================================*/
 /** Parameter des horizontalen Bereichs brechnen */
 //*==============================================*/

 private void scaleDef() {
 int maxValWidth = maxTextWidth( valCut, nfVal, valFormFont); // max. Weite der Y-AchsenBeschr.

  /*--------------------------------------------------------*/
  /* Platzbedarf der Markierungen auf der Y-Achse ermitteln */
  /*--------------------------------------------------------*/
  
  switch(xyForm & (LEFT + RIGHT)) {
   case LEFT:                       // Y-Achse liegt ganz rechts
         bRight  = bRight + markLen; // Markierungen nach ganz rechts
         bRight  = bRight + maxValWidth;
         bLeft   = bLeft  + defNameWidth;
         markVal = -1;
        break;
   case RIGHT:                      // Y-Achse liegt ganz links
         bLeft   = bLeft  + markLen;   // Markierungen nach ganz links
         bLeft   = bLeft  + maxValWidth;
         bRight  = bRight + defNameWidth;
         markVal = 1;
        break;
   case (LEFT + RIGHT):             // Y-Achse liegt im Ursprung

         /*----------------------------------------------*/
         /* in Abhngigkeit der Achsenlngen verschieben */
         /*----------------------------------------------*/

         if   (Math.abs(defMin) <= Math.abs(defMax)) // Y-Achse weiter rechts
              {
               bRight  = bRight + defNameWidth;
               markVal = 1;
              }
         else {
               bLeft   = bLeft + defNameWidth;
               markVal = -1;
              }
        break;
  }

  /*----------------------------------*/
  /* Platzbedarf der Pfeile ermitteln */
  /*----------------------------------*/

  if ((xyForm & RIGHT) == RIGHT) bRight = bRight + shaftLength + peakLen; // Pfeil nach rechts
  if ((xyForm & LEFT)  == LEFT)  bLeft  = bLeft  + shaftLength + peakLen; // Pfeil nach links

  xScale = (double)(size.width - (bRight + bLeft)) / (defMax - defMin);
 }

 //*====================================================*/
 /** X-Wert der physikalischen Ausgabeeinheit ermitteln */
 //*====================================================*/

 private int xScreen( double x) {
  return bLeft + (int)(xScale * (x - defMin) - 0.5);
 }

 //*============================================*/
 /** Parameter des vertikalen Bereichs brechnen */
 //*============================================*/

 private void scaleVal() {
 int valFormHeight = getTextHeight( valFormFont);

  /*--------------------------------------------------------*/
  /* Platzbedarf der Markierungen auf der X-Achse ermitteln */
  /*--------------------------------------------------------*/
  
  switch(xyForm & (UP + DOWN)) {
   case UP:                         // X-Achse liegt     ganz unten
         bLower = bLower + markLen; // Markierungen nach ganz unten
         bLower = bLower + valFormHeight;
         bUpper = bUpper + valNameHeight;
         markDef = -1;
        break;
   case DOWN:                       // X-Achse liegt     ganz oben
         bUpper = bUpper + markLen; // Markierungen nach ganz oben
         bUpper = bUpper + valFormHeight;
         bLower = bLower + valNameHeight;
         markDef = 1;
        break;
   case (UP + DOWN):                // X-Achse liegt im Ursprung

         /*----------------------------------------------*/
         /* in Abhngigkeit der Achsenlngen verschieben */
         /*----------------------------------------------*/

         if   (Math.abs(valMin) <= Math.abs(valMax)) // X-Achse weiter oben
              {
               bUpper = bUpper + valNameHeight;
               markDef = -1;
              }
         else {
               bLower = bLower + valNameHeight;
               markDef = 1;
              }

        break;
  }

  /*----------------------------------*/
  /* Platzbedarf der Pfeile ermitteln */
  /*----------------------------------*/

  if ((xyForm & UP)   == UP)    bUpper = bUpper + shaftLength + peakLen; // Pfeil nach oben
  if ((xyForm & DOWN) == DOWN)  bLower = bLower + shaftLength + peakLen; // Pfeil nach unten

  yScale = (double)(size.height - (bLower + bUpper)) / (valMax - valMin);
 }

 //*====================================================*/
 /** Y-Wert der physikalischen Ausgabeeinheit ermitteln */
 //*====================================================*/

 private int yScreen( double y) {
  return size.height - (bLower + (int)(yScale * (y - valMin) - 0.5)) - 1; // 1 Pixel Korrektur
 }

 //*================================================*/
 /** Position der X-Achse auf der Y-Achse ermitteln */
 //*================================================*/
 
 private int getXonY() {
 double retval = 0.0;

  switch(xyForm & (LEFT + RIGHT)) {
   case LEFT:           retval = defMax; break; // Y-Achse liegt ganz rechts
   case RIGHT:          retval = defMin; break; // Y-Achse liegt ganz links
   case (LEFT + RIGHT): retval = 0.0;    break; // Y-Achse liegt im Ursprung
  }
  return xScreen( retval);
 }

 //*================================================*/
 /** Position der Y-Achse auf der X-Achse ermitteln */
 //*================================================*/
 
 private int getYonX() {
 double retval = 0.0;

  switch(xyForm & (UP + DOWN)) {
   case UP:          retval = valMin; break; // X-Achse liegt ganz unten
   case DOWN:        retval = valMax; break; // X-Achse liegt ganz oben
   case (UP + DOWN): retval = 0.0;    break; // X-Achse liegt im Ursprung
  }
  return yScreen( retval);
 }

 //*=================*/
 /** Achsen zeichnen */
 //*=================*/

 private void drawAxes( int xfrom, int yfrom) {
  gc.drawLine( xScreen( defMin), yfrom, xScreen( defMax), yfrom);
  gc.drawLine( xfrom,  yScreen( valMin), xfrom,  yScreen( valMax));
 }

 //*=====================================*/
 /** Beschriften des Definitionsbereichs */
 //*=====================================*/

 private void drawDefNums( int yfrom) {
 int         yp      = yfrom;
 Font        actfont = gc.getFont();
 FontMetrics fm;
 double      val;
 String      str;
 int         width;
 
  gc.setFont( defFormFont);
  fm = gc.getFontMetrics();

  /*-------------------------------------------*/
  /* Test, ob ober- oder unterhalb der X-Achse */
  /*-------------------------------------------*/

  if   (markDef < 0)
       yp = yp + fm.getHeight();                                // marks down
  else yp = yp - (fm.getDescent() + fm.getLeading() + markLen); // marks down

  gc.setColor( defFormColor);
  for( int i=0; i<=xCut; i++) {
   val = defMin + defCut * i;
   str = nfDef.format( val);
   width = getTextWidth( str, defFormFont);
   gc.drawString( str, xScreen( val) - width / 2, yp);
  }
  gc.setColor( paintColor);
  gc.setFont( actfont);
 }

 //*===================================*/
 /** Aufteilen des Definitionsbereichs */
 //*===================================*/

 private void drawDefGraduation( int yfrom) {
 int xp;

  for( int i=0; i<=xCut; i++) {
   xp  = xScreen( defMin + defCut * i);
   gc.drawLine( xp, yfrom, xp, yfrom - markLen * markDef);
  }
 }

 //*===============================*/
 /** Beschriften des Wertebereichs */
 //*===============================*/

 private void drawValNums( int xfrom) {
 int         maxWidth = maxTextWidth( valCut, nfVal, valFormFont);
 int         xp       = xfrom - markLen * markVal;
 Font        actfont  = gc.getFont();
 int         xc       = 0;
 FontMetrics fm;
 double      val;
 int         yp;
 String      str;
 
  gc.setFont( valFormFont);
  fm = gc.getFontMetrics();
  
  gc.setColor( valFormColor);
  for( int i=0; i<=yCut; i++) {
   val = valMin + valCut * i;
   yp  = yScreen( val);
   str = nfVal.format( val);

  /*----------------------------------------*/
  /* Test, ob links oder rechts der Y-Achse */
  /*----------------------------------------*/

   if   (markVal < 0)
        {
         xc = maxWidth - fm.stringWidth( str);
         gc.drawString( str + " ", xp + xc, yp);
        }
   else {
         xc = fm.stringWidth( str + " ");
         gc.drawString( " " + str, xp - xc, yp);
        }
  }
  gc.setColor( paintColor);
  gc.setFont( actfont);
 }

 //*=============================*/
 /** Aufteilen des Wertebereichs */
 //*=============================*/

 private void drawValGraduation( int xfrom) {
 int         maxWidth = maxTextWidth( valCut, nfVal, valFormFont);
 int         xp       = xfrom - markLen * markVal;
 Font        actfont  = gc.getFont();
 FontMetrics fm;
 double      val;
 int         yp;
 int         xc;
 String      str;

  gc.setFont( valFormFont);
  fm = gc.getFontMetrics();
  
  for( int i=0; i<=yCut; i++) {
   val = valMin + valCut * i;
   yp  = yScreen( val);
   str = nfVal.format( val);
   
   if   ((xyForm & (LEFT + RIGHT)) == LEFT)
        xc = maxWidth - fm.stringWidth( str);
   else xc = - fm.stringWidth( str);
   gc.drawLine( xfrom, yp, xp, yp);
  }
  gc.setFont( actfont);
 }

 //*====================================*/
 /** Teilstriche an den Achsen zeichnen */
 //*====================================*/

 private void drawGraduationMarks( int xfrom, int yfrom) {
  drawDefGraduation( yfrom); // Teilung      des Definitionsbereichs
  drawValGraduation( xfrom); // Teilung      des Wertebereichs
 }

 //*===============================*/
 /** Werte der Teilungen schreiben */
 //*===============================*/

 private void drawGraduationValues( int xfrom, int yfrom) {
  drawDefNums( yfrom); // Beschriftung des Definitionsbereichs
  drawValNums( xfrom); // Beschriftung des Wertebereichs
 }

 //*===============================*/
 /** Pfeile am Achsenende zeichnen */
 //*===============================*/

 private void drawArrows( int xfrom, int yfrom) {
 int      xend = 0;
 int      yend = 0;
 Polygon  arr;

  //*-------------------------*/
  /** Pfeilrichtung festlegen */
  //*-------------------------*/

  if ((xyForm & RIGHT) == RIGHT)
     {
      arr  = new Polygon();
      xend = size.width - bRight + shaftLength;
      yend = yfrom;
      arr.addPoint( xend,       yend);
      arr.addPoint( xend,       yend - peakWidth);
      arr.addPoint( xend + peakLen, yend);
      arr.addPoint( xend,       yend + peakWidth);
      arr.addPoint( xend,       yend);
      gc.drawPolygon( arr);
      gc.fillPolygon( arr);
      gc.drawLine( xend, yend, xend - shaftLength, yend);
     }
  if ((xyForm & LEFT) == LEFT)
     {
      arr  = new Polygon();
      xend = bLeft - shaftLength;
      yend = yfrom;
      arr.addPoint( xend,       yend);
      arr.addPoint( xend,       yend - peakWidth);
      arr.addPoint( xend - peakLen, yend);
      arr.addPoint( xend,       yend + peakWidth);
      arr.addPoint( xend,       yend);
      gc.drawPolygon( arr);
      gc.fillPolygon( arr);
      gc.drawLine( xend, yend, xend + shaftLength, yend);
     }
  if ((xyForm & UP) == UP)
     {
      arr  = new Polygon();
      yend = bUpper - shaftLength;
      xend = xfrom;
      arr.addPoint( xend,         yend);
      arr.addPoint( xend - peakWidth, yend);
      arr.addPoint( xend,         yend - peakLen);
      arr.addPoint( xend + peakWidth, yend);
      arr.addPoint( xend,         yend);
      gc.drawPolygon( arr);
      gc.fillPolygon( arr);
      gc.drawLine( xend, yend, xend, yend + shaftLength);
     }
  if ((xyForm & DOWN) == DOWN)
     {
      arr  = new Polygon();
      yend = size.height - bLower + shaftLength;
      xend = xfrom;
      arr.addPoint( xend,         yend);
      arr.addPoint( xend - peakWidth, yend);
      arr.addPoint( xend,         yend + peakLen);
      arr.addPoint( xend + peakWidth, yend);
      arr.addPoint( xend,         yend);
      gc.drawPolygon( arr);
      gc.fillPolygon( arr);
      gc.drawLine( xend, yend, xend, yend - shaftLength);
     }
 }

 //*=============================================*/
 /** Bezeichnung des Definitionbereichs ausgeben */
 //*=============================================*/

 private void drawDefName( int xfrom, int yfrom) {
 Font        actfont = gc.getFont();
 FontMetrics fm;
 int         xp;

  gc.setFont( defNameFont);
  fm = gc.getFontMetrics();

  /*-----------------------------------------*/
  /* Test, ob positive oder negative X-Achse */
  /*-----------------------------------------*/

  gc.setColor( defNameColor);
  if   (markVal < 0)
       {
        xp = 0;
        gc.drawString( defName + " ", xp, yfrom);
       }
  else {
        xp = size.width - defNameWidth;
        gc.drawString( " " + defName, xp, yfrom);
       }
  gc.setColor( paintColor);
  gc.setFont( actfont);
 }
 
 //*========================================*/
 /** Bezeichnung des Wertebereichs ausgeben */
 //*========================================*/

 private void drawValName( int xfrom, int yfrom) {
 Font        actfont = gc.getFont();
 FontMetrics fm;
 int         yp;

  gc.setFont( valNameFont);
  fm = gc.getFontMetrics();

  /*-----------------------------------------*/
  /* Test, ob positive oder negative Y-Achse */
  /*-----------------------------------------*/

  if  (markDef < 0)
       yp = fm.getAscent();                                    // positive Achse
  else yp = size.height - (fm.getDescent() + fm.getLeading()); // negative Achse

  gc.setColor( valNameColor);
  gc.drawString( valName, xfrom - fm.stringWidth( valName) / 2, yp);
  gc.setColor( paintColor);
  gc.setFont( actfont);
 }
 
 //*==============================*/
 /** Achsenbezeichnungen ausgeben */
 //*==============================*/

 private void drawNames( int xfrom, int yfrom) {
  drawDefName( xfrom, yfrom);
  drawValName( xfrom, yfrom);
 }
 
 //*==============================================*/
 /** Darstellungsbereich leeren und Farben setzen */
 //*==============================================*/

 private void setColors() {
  gc.setColor( background);
  gc.fillRect( 0, 0, size.width, size.height);
  gc.setColor( paintColor);
 }

 /**************************************************************************/
 /** Kleinsten darstellbaren Unterschied des Definitionsbereichs bergeben */
 /**************************************************************************/
 
 public double getMinDefDelta() {
  return (defMax - defMin) / (double)(size.width - (bRight + bLeft));
 }
 
 /********************************************************************/
 /** Kleinsten darstellbaren Unterschied des Wertebereichs bergeben */
 /********************************************************************/
 
 public double getMinValDelta() {
  return (valMax - valMin) / (double)(size.height - (bLower + bUpper));
 }
 
 //************************/
 /** Grafikbereich leeren */
 //************************/

 public void clear() {
  varInit();   // Parameter initialisieren
  scaleDef();  // skalieren des Definitionsbereichs
  scaleVal();  // skalieren des Wertebereichs
  setColors(); // Darstellungsbereich leeren und Farben setzen
 }

 //*******************************/
 /** Koordinatenkreuz darstellen */
 //*******************************/

 public void drawCross() {
 int xony;
 int yonx;
 
  xony = getXonY();  // Position der X- auf der Y-Achse ermitteln
  yonx = getYonX();  // Position der Y- auf der X-Achse ermitteln

  drawAxes(             xony, yonx);  // Achsen zeichnen
  drawGraduationMarks(  xony, yonx);  // Teilung der Achsen zeichnen
  drawGraduationValues( xony, yonx);  // Werte der Teilungen ausgeben
  drawArrows(           xony, yonx);  // Pfeile zeichnen
  drawNames(            xony, yonx);  // Achsenbezeichnungen ausgeben
 }

 /*********************************************/
 /** Zeichnen mit den gemachten Einstellungen */
 /*********************************************/

 public void paint() {
  clear();
  drawCross();
 }
 
 //******************************************/
 /** absolutes zeichnen einer geraden Linie */
 //******************************************/

 public void line( double x0, double y0, double x1, double y1) {
  gc.drawLine( xScreen( x0), yScreen( y0), xScreen( x1), yScreen( y1));
 }

 //************************************/
 /** absolutes schreiben eines Textes */
 //************************************/

 public void write( String str, double xpos, double ypos) {
  gc.drawString( str, xScreen( xpos), yScreen( ypos));
 }

 //***************************************/
 /** Lnge der Markierungsstriche setzen */
 //***************************************/

 public void setMarkLen( int len) {
  markLen = len;
 }

 //******************************************/
 /** Lnge der Markierungsstriche bergeben */
 //******************************************/

 public int getMarkLen() {
  return markLen;
 }

 //***********************************************/
 /** Lnge des Pfeilstiels ohne Spitze festlegen */
 //***********************************************/

 public void setShaftLength( int len) {
  shaftLength = len;
 }

 //***********************************************/
 /** Lnge des Pfeilstiels ohne Spitze bergeben */
 //***********************************************/

 public int getShaftLength() {
  return shaftLength;
 }

 //******************************/
 /** Lnge der Spitze festlegen */
 //******************************/

 public void setPeakLen( int len) {
  peakLen = len;
 }
 
 //******************************/
 /** Lnge der Spitze bergeben */
 //******************************/

 public int getPeakLen() {
  return peakLen;
 }
 
 //*******************************/
 /** Breite der Spitze festlegen */
 //*******************************/

 public void setPeakWidth( int width) {
  peakWidth = width / 2;
 }

 //*******************************/
 /** Breite der Spitze bergeben */
 //*******************************/

 public int getPeakWidth() {
  return peakWidth * 2;
 }

 //*******************************************************/
 /** Hintergrundfarbe des Darstellungsbereichs festlegen */
 //*******************************************************/

 public void setBackground( Color col) {
  background = col;
 }

 //*******************************************************/
 /** Hintergrundfarbe des Darstellungsbereichs bergeben */
 //*******************************************************/

 public Color getBackground() {
  return background;
 }
 
 //*************************************************/
 /** Zeichenfarbe im Darstellungsbereich festlegen */
 //*************************************************/

 public void setPaintColor( Color col) {
  paintColor = col;
 }

 //*************************************************/
 /** Zeichenfarbe im Darstellungsbereich bergeben */
 //*************************************************/

 public Color getPaintColor() {
  return paintColor;
 }

 //***********************************************************/
 /** Farbe der Bezeichnung des Definitionsbereichs festlegen */
 //***********************************************************/

 public void setDefNameColor( Color col) {
  defNameColor = col;
 }
 
 //***********************************************************/
 /** Farbe der Bezeichnung des Definitionsbereichs bergeben */
 //***********************************************************/

 public Color getDefNameColor() {
  return defNameColor;
 }
 
 //************************************************************/
 /** Farbe der Beschriftung des Definitionsbereichs festlegen */
 //************************************************************/

 public void setDefFormColor( Color col) {
  defFormColor = col;
 }
 
 //************************************************************/
 /** Farbe der Beschriftung des Definitionsbereichs bergeben */
 //************************************************************/

 public Color getDefFormColor() {
  return defFormColor;
 }

 //*****************************************************/
 /** Farbe der Bezeichnung des Wertebereichs festlegen */
 //*****************************************************/

 public void setValNameColor( Color col) {
  valNameColor = col;
 }
 
 //*****************************************************/
 /** Farbe der Bezeichnung des Wertebereichs bergeben */
 //*****************************************************/

 public Color getValNameColor() {
  return valNameColor;
 }
 
 //******************************************************/
 /** Farbe der Beschriftung des Wertebereichs festlegen */
 //******************************************************/

 public void setValFormColor( Color col) {
  valFormColor = col;
 }

 //******************************************************/
 /** Farbe der Beschriftung des Wertebereichs bergeben */
 //******************************************************/

 public Color getValFormColor() {
  return valFormColor;
 }

 //*******************************************/
 /** Bezeichnung des Wertebereichs festlegen */
 //*******************************************/

 public void setValName( String name) {
  valName = name;
 }

 //*******************************************/
 /** Bezeichnung des Wertebereichs bergeben */
 //*******************************************/

 public String getValName() {
  return valName;
 }
 
 //**************************************************/
 /** Font des Namens fr den Wertebereich festlegen */
 //**************************************************/

 public void setValNameFont( String name, int style, int size) {
  valNameFont = new Font( name, style, size);
 }

 //**************************************************/
 /** Font des Namens fr den Wertebereich bergeben */
 //**************************************************/

 public Font getValNameFont() {
  return valNameFont;
 }

 //*****************************************************************************************/
 /** Bezeichnung des Definitionsbereichs festlegen und Wertebereichs als Funktion benennen */
 //*****************************************************************************************/

 public void setDefValName( String name) {
  defName = name;
  setValName( "f(" + name + ")");
 }

 //*************************************************/
 /** Bezeichnung des Definitionsbereichs festlegen */
 //*************************************************/

 public void setDefName( String name) {
  defName = name;
 }

 //*************************************************/
 /** Bezeichnung des Definitionsbereichs bergeben */
 //*************************************************/

 public String getDefName() {
  return defName;
 }
 
 //********************************************************/
 /** Font des Namens fr den Definitionsbereich festlegen */
 //********************************************************/

 public void setDefNameFont( String name, int style, int size) {
  defNameFont = new Font( name, style, size);
 }

 //********************************************************/
 /** Font des Namens fr den Definitionsbereich bergeben */
 //********************************************************/

 public Font getDefNameFont() {
  return defNameFont;
 }

 //********************************/
 /** Definitionsbereich festlegen */
 //********************************/

 public void setDefArea( double min, double max) {
  defMin = min;
  defMax = max;
 }

 //****************************************************/
 /** minimalen Wert des Definitionsbereichs bergeben */
 //****************************************************/

 public double getMinDefArea() {
  return defMin;
 }
 
 //****************************************************/
 /** maximalen Wert des Definitionsbereichs bergeben */
 //****************************************************/

 public double getMaxDefArea() {
  return defMax;
 }
 
 //**************************************************/
 /** Zahlenformat des Definitionsbereichs festlegen */
 //**************************************************/

 public void setDefFormat( String form) {
  formDef = form;
  nfDef = new DecimalFormat( formDef);
 }

 //**************************************************/
 /** Zahlenformat des Definitionsbereichs bergeben */
 //**************************************************/

 public DecimalFormat getDefFormat() {
  return nfDef;
 }

 //**************************************************/
 /** Font der Werte im Definitionsbereich festlegen */
 //**************************************************/

 public void setDefFormFont( String name, int style, int size) {
  defFormFont = new Font( name, style, size);
 }

 //**************************************************/
 /** Font der Werte im Definitionsbereich bergeben */
 //**************************************************/

 public Font getDefFormFont() {
  return defFormFont;
 }

 //**************************/
 /** Wertebereich festlegen */
 //**************************/

 public void setValArea( double min, double max) {
  valMin = min;
  valMax = max;
 }

 //*********************************************/
 /** minimalen Wert des Wertebereich bergeben */
 //*********************************************/

 public double getMinValArea() {
  return valMin;
 }
 
 //*********************************************/
 /** maximalen Wert des Wertebereich bergeben */
 //*********************************************/

 public double getMaxValArea() {
  return valMax;
 }
 
 //********************************************/
 /** Zahlenformat des Wertebereichs festlegen */
 //********************************************/

 public void setValFormat( String form) {
  formVal = form;
  nfVal = new DecimalFormat( formVal);
 }

 //********************************************/
 /** Zahlenformat des Wertebereichs bergeben */
 //********************************************/

 public DecimalFormat getValFormat() {
  return nfVal;
 }

 //********************************************/
 /** Font der Werte im Wertebereich festlegen */
 //********************************************/

 public void setValFormFont( String name, int style, int size) {
  valFormFont = new Font( name, style, size);
 }

 //********************************************/
 /** Font der Werte im Wertebereich bergeben */
 //********************************************/

 public Font getValFormFont() {
  return valFormFont;
 }

 //*********************************************/
 /** Teilung des Definitionsbereichs festlegen */
 //*********************************************/

 public void setDefCut( int cuts) {
  xCut = cuts;
 }
 
 //*********************************************/
 /** Teilung des Definitionsbereichs bergeben */
 //*********************************************/

 public int getDefCut() {
  return xCut;
 }

 //***************************************/
 /** Teilung des Wertebereichs festlegen */
 //***************************************/

 public void setValCut( int cuts) {
  yCut = cuts;
 }

 //***************************************/
 /** Teilung des Wertebereichs bergeben */
 //***************************************/

 public int getValCut() {
  return yCut;
 }

 //*************************************************/
 /** Referenz auf einen Graphic Context herstellen */
 //*************************************************/

 public void setGraphicContext( Graphics g) {
  gc = g;
 }

 //*****************************/
 /** Graphic Context bergeben */
 //*****************************/

 public Graphics getGraphicContext() {
  return gc;
 }

 //********************************************************/
 /** Referenz auf die Dimensionierung des Graphic Context */
 //********************************************************/

 public void setSize( Dimension dim) {
  size = dim;
 }

 //*************************************************/
 /** Dimensionierung des Graphic Context bergeben */
 //*************************************************/

 public Dimension setSize() {
  return size;
 }

 //**********************************************/
 /** CREATOR ohne Bindung an grafischen Kontext */
 //**********************************************/

 public Mathdisp() {
 }

 //*********************************************/
 /** CREATOR mit Bindung an grafischen Kontext */
 //*********************************************/

 public Mathdisp( Graphics g) {
  setGraphicContext( g);
 }

 //**************************************************************/
 /** CREATOR mit Bindungen an grafischen Kontext und Ausdehnung */
 //**************************************************************/

 public Mathdisp( Graphics g, Dimension dim) {
  setGraphicContext( g);
  setSize( dim);
 }
}
