
package gui;

//*************************************/
//*                                   */
/**  Klasse eines praktischen Panels  */
//*                                   */
//*************************************/

public class HandyPanel extends java.awt.Panel {

/*-------------------------------------------------------*/
/* mgliche Eigenschaften des perspektivischen Aussehens */
/*-------------------------------------------------------*/

/** Synonym fr flache Darstellung */
public final static byte FLAT =  0;

/** Synonym fr gravierende Darstellung */
public final static byte GRAVE =  1;

/** Synonym fr erhabene Darstellung */
public final static byte ELEVATE  =  2;

/*-------------------------------------------------*/
/* mgliche Erweiterungen des farblichen Aussehens */
/*-------------------------------------------------*/

/** Farbe etwas dunkler */
public final static byte DARKER =  4;

/** Farbe etwas heller */
public final static byte BRIGHTER =  8;

/*--------*/
/* Rahmen */
/*--------*/

/** Rahmen sichtbar */
public final static byte BORDER = 16;

/*-------------------------*/
/* farbliche Eigenschaften */
/*-------------------------*/

/** Farbe des Panels */
private java.awt.Color col = java.awt.Color.lightGray;

/** Farbe des Rahmens */
private java.awt.Color bordcol = java.awt.Color.lightGray;

/*-----------------------*/
/* Initialisierungswerte */
/*-----------------------*/

/** perspektivische Eigenschaften */
private int how = FLAT;

/** Rahmenbreite */
private int thick = 1;

/** Abstand zum Rahmen */
private int allins = 5;

 //***********************************/
 /** Umfang des Panels neu festlegen */
 //***********************************/

 public java.awt.Insets getInsets() {
  return new java.awt.Insets( allins, allins, allins, allins);
 }

 //****************************************/
 /** Darstellung des Panels neu festlegen */
 //****************************************/

 public void paint( java.awt.Graphics g) {

  /*---------------------------*/
  /* Test, ob Panel Rahmen hat */
  /*---------------------------*/

  if   ((this.how & BORDER) != 0) 
       {

        /*-------------------------------------------*/
        /* Test, ob Rahmen erhaben oder graviert ist */
        /*-------------------------------------------*/

        if ((this.how & (ELEVATE + GRAVE)) != 0)
           {
            g.setColor( this.bordcol);
            g.fill3DRect( 0, 0,
                          getBounds().width,
                          getBounds().height,
                          (this.how & ELEVATE) != 0);

            g.fillRect( 1, 1,
                        getBounds().width -2,
                        getBounds().height-2);

            g.fill3DRect( this.thick, this.thick,
                          getBounds().width  - (this.thick << 1),
                          getBounds().height - (this.thick << 1),
                          (this.how & GRAVE) != 0);

            g.setColor( this.getBackground());
            g.fillRect( this.thick+1, this.thick+1,
                        getBounds().width  - (this.thick << 1)-2,
                        getBounds().height - (this.thick << 1)-2);
            return;
           }

        /*---------------------------------------------------*/
        /* Test, ob einfacher Rahmen dunkler oder heller ist */
        /*---------------------------------------------------*/

        g.setColor( this.bordcol);
        if   ((this.how & DARKER) != 0)
             g.setColor( this.bordcol.darker());
        else if ((this.how & BRIGHTER) != 0)
                g.setColor( this.bordcol.brighter());

        /*-----------------*/
        /* Rahmen zeichnen */
        /*-----------------*/

        g.fillRect( 0, 0, 
                    this.thick, getBounds().height);
        g.fillRect( 0, getBounds().height-this.thick,
                    getBounds().width, getBounds().height);
        g.fillRect( 0, 0, 
                    getBounds().width, this.thick);
        g.fillRect( getBounds().width-this.thick, 0, 
                    getBounds().width, getBounds().height);

        g.setColor( this.getBackground());
        g.fillRect( this.thick+1, this.thick+1,
                    getBounds().width  - (this.thick << 1)-2,
                    getBounds().height - (this.thick << 1)-2);
        return;
   }

  /*------------------------------------------*/
  /* Test, ob Panel erhaben oder graviert ist */
  /*------------------------------------------*/

  if ((this.how & (ELEVATE + GRAVE)) != 0)
     {
      g.setColor( this.bordcol);
      g.fill3DRect( 0, 0,
                    getBounds().width,
                    getBounds().height,
                    (this.how & ELEVATE) != 0);

      g.setColor( this.getBackground());
      g.fillRect( this.thick+1, this.thick+1,
                  getBounds().width  - (this.thick << 1)-1,
                  getBounds().height - (this.thick << 1)-1);
      return;
     }
 }

 //*=======================*/
 /** Default-Farben setzen */
 //*=======================*/

 private void defaultColor() {
  this.setBackground( java.awt.Color.lightGray);
  this.col = this.getBackground();
  this.bordcol = this.col;
 }

 //******************************/
 /** Farbe des Panels festlegen */
 //******************************/

 public void setColor( java.awt.Color color) {
  this.col = color;
  this.setBackground( this.col);
 }

 //******************************/
 /** Farbe des Panels bergeben */
 //******************************/

 public java.awt.Color getColor() {
  return this.col;
 }

 //************************************/
 /** Farbe des Panelrahmens festlegen */
 //************************************/

 public void setBorderColor( java.awt.Color color) {
  this.bordcol = color;
 }

 //************************************/
 /** Farbe des Panelrahmens festlegen */
 //************************************/

 public java.awt.Color getBorderColor() {
  return this.bordcol;
 }

 //*************************************/
 /** Breite des Panelrahmens festlegen */
 //*************************************/

 public void setBorderWidth( int width) {
  this.thick = thick;
 }

 //*****************************************************/
 /** Dreidimesionale Eigenschaften des Panes festlegen */
 //*****************************************************/

 public void set3D( int persp) {
  this.how = persp;
 }

 //****************************************/
 /** Einfaches, flaches Panel in Hellgrau */
 //****************************************/

 public HandyPanel() {
  super();
  this.defaultColor();
 }

 //************************************/
 /** Flaches Panel in gewhlter Farbe */
 //************************************/

 public HandyPanel( java.awt.Color col) {
  super();

  /*--------------------------------------*/
  /* Insets auf die Rahmenbreite beziehen */
  /*--------------------------------------*/

  if ((this.how & BORDER) != 0) this.allins += this.thick;

  this.bordcol = java.awt.Color.lightGray;
  this.col = col;
  this.setBackground( this.col);
 }

 //*************************************/
 /** Panel mit gewhlten Eigenschaften */
 //*************************************/

 public HandyPanel( int how) {
  super();
  this.how   = how;

  /*--------------------------------------*/
  /* Insets auf die Rahmenbreite beziehen */
  /*--------------------------------------*/

  if ((this.how & BORDER) != 0) this.allins += this.thick;

  this.defaultColor();
 }

 //***********************************************/
 /** Panel mit gewhlten Eigenschaften und Farbe */
 //***********************************************/

 public HandyPanel( int how, java.awt.Color col) {
  super();
  this.how = how;

  /*--------------------------------------*/
  /* Insets auf die Rahmenbreite beziehen */
  /*--------------------------------------*/

  if ((this.how & BORDER) != 0) this.allins += this.thick;

  this.bordcol = java.awt.Color.lightGray;
  this.col = col;
  this.setBackground( this.col);
 }

 //************************************************************************/
 /** Panel mit gewhlten Eigenschaften, Farbe und ergnzender Rahmenfarbe */
 //************************************************************************/

 public HandyPanel( int how, java.awt.Color col, java.awt.Color bordcol) {
  super();
  this.how = how;

  /*--------------------------------------*/
  /* Insets auf die Rahmenbreite beziehen */
  /*--------------------------------------*/

  if ((this.how & BORDER) != 0) this.allins += this.thick;

  this.col     = col;
  this.bordcol = bordcol;
  this.setBackground( this.col);
 }

 //************************************************************************/
 /** Panel mit gewhlten Eigenschaften, Farbe und ergnzender Rahmenfarbe */
 //************************************************************************/

 public HandyPanel( int how, int thick) {
  super();
  this.how   = how;
  this.thick = thick;

  /*--------------------------------------*/
  /* Insets auf die Rahmenbreite beziehen */
  /*--------------------------------------*/

  if ((this.how & BORDER) != 0) this.allins += this.thick;

  this.defaultColor();
 }

 //***************************************************************************/
 /** Panel mit gewhlten Eigenschaften, Farbe und individueller Rahmenbreite */
 //***************************************************************************/

 public HandyPanel( int how, int thick, java.awt.Color col) {
  super();
  this.how   = how;
  this.thick = thick;

  /*--------------------------------------*/
  /* Insets auf die Rahmenbreite beziehen */
  /*--------------------------------------*/

  if ((this.how & BORDER) != 0) this.allins += this.thick;

  this.bordcol = java.awt.Color.lightGray;
  this.col = col;
  this.setBackground( this.col);
 }

 //**************************************************************************************/
 /** Panel mit gewhlten Eigenschaften, Farbe und individueller Rahmenbreite und -farbe */
 //**************************************************************************************/

 public HandyPanel( int how, int thick, java.awt.Color col, java.awt.Color bordcol) {
  super();
  this.how   = how;
  this.thick = thick;

  /*--------------------------------------*/
  /* Insets auf die Rahmenbreite beziehen */
  /*--------------------------------------*/

  if ((this.how & BORDER) != 0) this.allins += this.thick;

  this.col     = col;
  this.bordcol = bordcol;
  this.setBackground( this.col);
 }
}
