
package mathpack.arithmetic;

import mathpack.grammar.*;
import mathpack.functions.*;
import mathpack.operators.*;

//*************************************/
//*                                   */
/** Klasse der elementaren Arithmetik */
//*                                   */
//*************************************/

public class Arithmetic {

/** koordinierende Grammatik */
private Grammar grammar = null;

 //*======================*/
 /** Operatoren festlegen */
 //*======================*/

 private void setOperators() {
  this.grammar.getOperators().addOperator( new Operator( '(', 1, null));
  this.grammar.getOperators().addOperator( new Operator( ')', 1, null));
  this.grammar.getOperators().addOperator( new Operator( '<', 2, new less()));
  this.grammar.getOperators().addOperator( new Operator( '>', 2, new greater()));
  this.grammar.getOperators().addOperator( new Operator( '=', 2, new equal()));
  this.grammar.getOperators().addOperator( new Operator( '|', 2, new disjugate()));
  this.grammar.getOperators().addOperator( new Operator( '+', 2, new add()));
  this.grammar.getOperators().addOperator( new Operator( '-', 2, new subtract()));
  this.grammar.getOperators().addOperator( new Operator( '&', 3, new conjugate()));
  this.grammar.getOperators().addOperator( new Operator( '*', 3, new multiply()));
  this.grammar.getOperators().addOperator( new Operator( '/', 3, new divide()));
  this.grammar.getOperators().addOperator( new Operator( '^', 4, new power()));
  this.grammar.getOperators().addOperator( new Operator( '!', 4, null));
 }

 //*======================*/
 /** Funktionen festlegen */
 //*======================*/

 private void setFunctions() {
  this.grammar.getFunctions().addFunction( new Function( "not",  new not()));
  this.grammar.getFunctions().addFunction( new Function( "abs",  new abs()));
  this.grammar.getFunctions().addFunction( new Function( "cos",  new cos()));
  this.grammar.getFunctions().addFunction( new Function( "cot",  new cot()));
  this.grammar.getFunctions().addFunction( new Function( "exp",  new exp()));
  this.grammar.getFunctions().addFunction( new Function( "ln",   new ln()));
  this.grammar.getFunctions().addFunction( new Function( "sin",  new sin()));
  this.grammar.getFunctions().addFunction( new Function( "sqrt", new sqrt()));
  this.grammar.getFunctions().addFunction( new Function( "tan",  new tan()));
 }

 /************************/
 /** Grammatik festlegen */
 /************************/

 public void setGrammar( Grammar grammar) {
  this.grammar = grammar;
 }
 
 /************************/
 /** Grammatik bergeben */
 /************************/

 public Grammar getGrammar() {
  return this.grammar;
 }

 //**************************************/
 /** Eine eigene Grammatik wird erzeugt */
 //**************************************/

 public Arithmetic() {
  this.grammar = new Grammar( new Operators(), new Functions());
  this.grammar.setValueCreator( new mathpack.arithmetic.value());      
  setOperators();
  setFunctions();
 }

 //**************************************************************/
 /** Eine vorhanden Grammatik wird verwendet und ggf. erweitert */
 //**************************************************************/

 public Arithmetic( Grammar gram) {
  this.grammar = gram;  
  this.grammar.setValueCreator( new mathpack.arithmetic.value());  
  setOperators();
  setFunctions();
 }
}
