
package mathpack.expression;

import mathpack.translate.*;
import mathpack.arithmetic.*;
import mathpack.operators.*;
import mathpack.functions.*;
import mathpack.constants.*;
import mathpack.variables.*;
import mathpack.grammar.*;
import java.util.*;

//***********************************/
//*                                 */
/** Klasse mathematischer Ausdrcke */
//*                                 */
//***********************************/

public class Expression {

/** Ausdruck in arithmetischer Notation */
private String expr = null;

/** Ausdruck in postfix Notation */
private String postfix = null;

/** die koordinierende Grammatik */
private Grammar grammar = null;

/** Datenstack */
private Stack stack = null;

/** Ergebnis der Berechnung, wenn temporrer Datenstack */
private Object result;

/*--------------------------*/
/* Fehlercodes fr Klammern */
/*--------------------------*/

/** interner Fehlercode */
private int errcode = 0;

/** Synonym fr keinen gefundenen Fehler */
public static final int OK = 0;

/** Synonym fr fehlende rechte Klammer */
public static final int RGTPARA = 1;

/** Synonym fr fehlende linke Klammer */
public static final int LFTPARA = 2;

/** Synonym fr Syntaxfehler */
public static final int SYNTAX = 4;

 //*===================================*/
 /** Test, ob Klammern ausgewogen sind */
 //*===================================*/

 private int testPara( String str) {
 int  idx = 0;
 int  cnt = 0;
 char chr = '@';

  /*-----------------*/
  /* Klammern zhlen */
  /*-----------------*/

  while( (idx < str.length()) && (cnt >= 0)) {
   chr = str.charAt( idx++);
   switch( chr) {
    case '(': cnt++; break;
    case ')': cnt--; break;
   }
  }

  /*----------------------*/
  /* Fehlercode bergeben */
  /*----------------------*/

  if (cnt < 0) return LFTPARA;
  if (cnt > 0) return RGTPARA;
  return OK;
 }

 //*===============================*/
 /** Wort aus UPN-String isolieren */
 //*===============================*/

 private String word( String str) {
 StringTokenizer tok = new StringTokenizer( str);

  return tok.nextToken().toString();
 }

 //*==========================================*/
 /** einzelne Zeichen des Elements berprfen */
 //*==========================================*/

 private boolean elementOk( String elm) {
 int     len    = elm.length();
 int     idx    = 0;
 boolean retval = true;
 char    ch;
 
  while( (idx < len) && retval) {
   ch = elm.charAt( idx);
   if  (idx == 0)
        retval = Character.isLetterOrDigit( ch);
   else retval = Character.isLetter( ch);
   idx++;
  }
  return retval;
 }

 //*==============================*/
 /** einzelnes Element berprfen */
 //*==============================*/

 private int testElement( String elm) {
        if (grammar.isValuable( elm)) return 0;
   else if (grammar.isOperator( elm)) return 0;
   else if (grammar.isFunction( elm)) return 0;
   else if (grammar.isConstant( elm)) return 0;
   else if (grammar.isVariable( elm)) return 0;
   else if (elementOk(          elm)) return 1;
   return -1;
 }
 
 //*******************************/
 /** Postfix-Ausdruck berprfen */
 //*******************************/

 private int testPostfix( String upnexp) {
 String expr   = new String( upnexp);
 int    retval = 0;
 String elm;

  while( (expr.length() > 0) && (retval >= 0)) {
   elm    = word( expr);
   expr   = expr.substring( elm.length());
   expr   = expr.trim();
   retval = testElement( elm);
  }
  return retval;
 }

 //*********************************************/
 /** Elemente des Postfix-Ausdrucks berprfen */
 //*********************************************/

 public String testElements( String upnexp) {
 String expr   = new String( upnexp);
 int    tstval = 0;
 String elm    = null;

  while( (expr.length() > 0) && (tstval == 0)) {
   elm    = word( expr);
   expr   = expr.substring( elm.length());
   expr   = expr.trim();
   tstval = testElement( elm);
  }
  if (tstval != 0) return elm;
  return null;
 }

 //***********************/
 /** Grammatik festlegen */
 //***********************/

 public void setGrammar( Grammar grammar) {
  this.grammar = grammar;
 }
 
 //***********************/
 /** Grammatik bergeben */
 //***********************/

 public Grammar getGrammar() {
  return this.grammar;
 }
 
 //**********************/
 /** Ausdruck festlegen */
 //**********************/

 public void setExpression( String expr) {
  this.expr    = expr;
  this.errcode = testPara( this.expr);
 }

 //**********************/
 /** Ausdruck bergeben */
 //**********************/

 public String getExpression() {
  return this.expr;
 }

 //******************************/
 /** Postfix-Ausdruck bergeben */
 //******************************/

 public String getPostfix() {
  return this.postfix;
 }

 //*****************************************/
 /** Ausdruck in postfix-Notation umformen */
 //*****************************************/

 public void translate() {
 Translate trans  = new Translate( this.grammar);
 String    locstr = trans.intoPostfix( this.expr);

  this.errcode = OK;
  if  (testPostfix( locstr) >= 0)
       this.postfix = locstr;
  else this.errcode = SYNTAX;
 }

 //************************************************************************/
 /** Postfix-Ausdruck berechnen. Evtl. temporren Datenstack instanzieren */
 //************************************************************************/

 public void calculate() {
 String upnexp = this.postfix;
 String elm;
 boolean temp = false;
 
  /*------------------------------------------------------------*/
  /* Test, ob ein temporrer Datenstack instanziert werden muss */
  /*------------------------------------------------------------*/

  temp = (stack == null);
  if (temp) stack = new Stack();

  while( upnexp.length() > 0) {
   elm = word( upnexp);
   upnexp = upnexp.substring( elm.length());
   upnexp = upnexp.trim();

        if (grammar.isValuable( elm)) grammar.createValue( elm).doIt( stack);
   else if (grammar.isOperator( elm)) grammar.getOperator( elm).doIt( stack);
   else if (grammar.isFunction( elm)) grammar.getFunction( elm).doIt( stack);
   else if (grammar.isConstant( elm)) grammar.getConstant( elm).doIt( stack);
   else if (grammar.isVariable( elm)) grammar.getVariable( elm).doIt( stack);
   else                       System.out.println( elm + " ist nicht definiert!!!");
  }

  /*-----------------------------------------------------*/
  /* Test, ob temporrer Datenstack entfernt werden muss */
  /*-----------------------------------------------------*/

  if (temp) 
     {
      this.result = stack.pop();
      stack = null;
     }
 }

 //*********************/
 /** Ergebis bergeben */
 //*********************/

 public Object getResult() {
  if  (stack != null)
       return stack.pop();
  else return result;
 }

 //**********************************/
 /** Menge der Konstanten festlegen */
 //**********************************/

 public void setConstants( Constants constants) {
  this.grammar.setConstants( constants);
 }

 //*********************************/
 /** einzelne Konstante hinzufgen */
 //*********************************/

 public void addConstant( Constant constant) {
 Constants constants = this.grammar.getConstants();

  if (constants == null)
     {
      setConstants( new Constants());
      constants = this.grammar.getConstants();
     }
  constants.addConstant( constant);
 }

 //*********************************/
 /** Menge der Variablen festlegen */
 //*********************************/

 public void setVariables( Variables variables) {
  this.grammar.setVariables( variables);
 }

 //********************************/
 /** einzelne Variable hinzufgen */
 //********************************/

 public void addVariable( Variable variable) {
 Variables variables = this.grammar.getVariables();

  if (variables == null)
     {
      setVariables( new Variables());
      variables = this.grammar.getVariables();
     }
  variables.addVariable( variable);
 }

 //************************/
 /** Datenstack festlegen */
 //************************/

 public void setStack( Stack stack) {
  this.stack = stack;
 }
 
 //************************/
 /** Datenstack bergeben */
 //************************/

 public Stack getStack() {
  return this.stack;
 }
 
 /*******************************************************************/
 /** Ausdruck ohne Grammatik, Konstanten und Variablen instanzieren */
 /*******************************************************************/

 public Expression() {
 }
 
 /****************************************/
 /** Ausdruck mit Grammatik instanzieren */
 /****************************************/

 public Expression( Grammar grammar) {
  this.grammar = grammar;
 }

 /******************************************************************/
 /** Ausdruck mit Grammatik, Konstanten und Variablen instanzieren */
 /******************************************************************/

 public Expression( Grammar grammar, Constants constants, Variables variables) {
  grammar.setConstants( constants);
  grammar.setVariables( variables);
  this.grammar = grammar;
 }
}
