package mathpack.grammar;

import mathpack.operators.*;
import mathpack.functions.*;
import mathpack.constants.*;
import mathpack.variables.*;

import mathpack.values.*;

//*************************************/
//*                                   */
/** Klasse der elementaren Arithmetik */
//*                                   */
//*************************************/

public class Grammar {

/** Menge aller Operatoren innerhalb der Grammatik */
private Operators operators = null;

/** Menge aller Funktionen innerhalb der Grammatik */
private Functions functions = null;

/** Menge aller Konstanten innerhalb der Grammatik */
private Constants constants = null;

/** Menge aller Variablen innerhalb der Grammatik */
private Variables variables = null;

/** Einzelnes Wertobjekt instanzieren */
private mathpack.values.anyValue mk;

 //**************************/
 /** Test, ob Zahl vorliegt */
 //**************************/

 public boolean isValuable( String valstr) {
  return mk.t( valstr);
 }

 //*************************************/
 /** Neues Werteobjekt (Zahl) erzeugen */
 //*************************************/

 public valueObject createValue( String valstr) {
  return new valueObject( mk.c( valstr));
 }

 //************************************************************/
 /** Festlegen der Klassifizierung aller Wertobjekte (Zahlen) */
 //************************************************************/

 public void setValueCreator( mathpack.values.anyValue creator) {
  mk = creator;
 }
 
 //**********************************/
 /** Menge der Opeartoren festlegen */
 //**********************************/

 public void setOperators( Operators opers) {
  this.operators = opers;
 }
 
 //**********************************/
 /** Menge der Opeartoren bergeben */
 //**********************************/

 public Operators getOperators() {
  return this.operators;
 }
 
 //**********************************************/
 /** einzelnen (ordinierten) Operator bergeben */
 //**********************************************/

 public Operator getOperator( int idx) {
  return ((Operator)operators.elementAt( idx));
 }
 
 //********************************/
 /** Benannten Operator bergeben */
 //********************************/

 public Operator getOperator( String name) {
 int idx = operators.findOperator( name.charAt( 0));

  if (idx < 0) return null;
  return ((Operator)operators.elementAt( idx));
 }

 //**********************************/
 /** Menge der Funktionen festlegen */
 //**********************************/

 public void setFunctions( Functions funcs) {
  this.functions = funcs;
 }
 
 //**********************************/
 /** Menge der Funktionen bergeben */
 //**********************************/

 public Functions getFunctions() {
  return this.functions;
 }
 
 //********************************************/
 /** einzelne (ordinierte) Funktion bergeben */
 //********************************************/

 public Function getFunction( int idx) {
  return ((Function)functions.elementAt( idx));
 }
 
 //*******************************/
 /** Benannte Funktion bergeben */
 //*******************************/

 public Function getFunction( String name) {
 int idx = -1;

  if( !name.startsWith( "_")) return null;           // Function Dummy testen
  idx = functions.findFunction( name.substring( 1)); // Function Dummy entfernen
  if (idx < 0) return null;
  return ((Function)functions.elementAt( idx));
 }

 //**********************************/
 /** Menge der Konstanten festlegen */
 //**********************************/

 public void setConstants( Constants consts) {
  this.constants = consts;
 }
 
 //**********************************/
 /** Menge der Konstanten bergeben */
 //**********************************/

 public Constants getConstants() {
  return this.constants;
 }
 
 //*********************************************/
 /** einzelne (ordinierte) Konstante bergeben */
 //*********************************************/

 public Constant getConstant( int idx) {
  return ((Constant)constants.elementAt( idx));
 }
 
 //********************************/
 /** Benannte Konstante bergeben */
 //********************************/

 public Constant getConstant( String name) {
 int idx = constants.findConstant( name);

  if (idx < 0) return null;
  return ((Constant)constants.elementAt( idx));
 }

 //*********************************/
 /** Menge der Variablen festlegen */
 //*********************************/

 public void setVariables( Variables vars) {
  this.variables = vars;
 }
 
 //*********************************/
 /** Menge der Variablen bergeben */
 //*********************************/

 public Variables getVariables() {
  return this.variables;
 }
 
 //********************************************/
 /** einzelne (ordinierte) Variable bergeben */
 //********************************************/

 public Variable getVariable( int idx) {
  return ((Variable)variables.elementAt( idx));
 }
 
 //*******************************/
 /** Benannte Variable bergeben */
 //*******************************/

 public Variable getVariable( String name) {
 int idx = variables.findVariable( name);

  if (idx < 0) return null;
  return ((Variable)variables.elementAt( idx));
 }

 //******************************/
 /** Test, ob Operator vorliegt */
 //******************************/

 public boolean isOperator( String str) {
  if(operators != null) return (getOperator( str) != null);
  return false;
 }

 //******************************/
 /** Test, ob Funktion vorliegt */
 //******************************/

 public boolean isFunction( String str) {
  if(functions != null) return (getFunction( str) != null);
  return false;
 }

 //*******************************/
 /** Test, ob Konstante vorliegt */
 //*******************************/

 public boolean isConstant( String str) {
  if(constants != null) return (getConstant( str) != null);
  return false;
 }

 //******************************/
 /** Test, ob Variable vorliegt */
 //******************************/

 public boolean isVariable( String str) {
  if(variables != null) return (getVariable( str) != null);
  return false;
 }

 /**********************************************************/
 /** Grammatik ohne Operatoren und Funktionen instanzieren */
 /**********************************************************/

 public Grammar() {
 }

 /**************************************************************************/
 /** Grammatik mit (vordefinierten) Operatoren und Funktionen instanzieren */
 /**************************************************************************/

 public Grammar( Operators opers, Functions funcs) {
  this.operators = opers;
  this.functions = funcs;
 }
}
