package mathpack.translate;

import mathpack.grammar.*;
import java.util.*;

//**************************************************/
//*                                                */
/**  Klasse fr Umwandlung von Infix nach Postfix  */
//*                                                */
//**************************************************/

public class Translate {

/*-------------------------*/
/* Fehlercode und Position */
/*-------------------------*/

/** enthlt Fehlercode (0=ok) */
private int error  = 0;

/** enthlt Fehlerposition (-1=ok) */
private int errpos = -1;

/** wird bentigt, um Fehlerposition zu bestimmen */
private String errstr = "";

/*----------------------------------------------------------------------*/
/* Synonyme zur Spezifizierung der mglichen Ergebnisse von Fehlertests */
/*----------------------------------------------------------------------*/

/** Synonym fr Syntaxfehler */
private static final int SYNTAX  = 3;

/** Synonym fr fehlende linke Klammer */
// private static final int LFTPARA = 2;

/** Synonym fr fehlende rechte Klammer */
// private static final int RGTPARA = 1;

/** Synonym fr keinen gefundenen Fehler */
private static final int TEST_OK = 0;

/*---------------------------*/
/* Funktionen und Operatoren */
/*---------------------------*/

/** Grammatik als Verallgemeinerung aller zulssigen Operatoren und Funktionen */
private Grammar grammar;

/*-----------------------------------*/
/* Dreh- und Angelpunkt der Wandlung */
/*-----------------------------------*/

/** Synonym des Environmets bei symbolischer Verarbeitung */
private StringBuffer envir = new StringBuffer();


 //*==============================================================*/
 /** jeweils erstes Element aus arithmetischem Ausdruck bergeben */
 //*==============================================================*/

 private String isolateElm( String str) {
 int idx = 0;

  if (str.length() < 1) return "";

  str = str.trim();
  while( (idx < str.length()) && (grammar.getOperators().findOperator( str.charAt( idx)) < 0))
   idx++;
   
  if (idx < 1) idx = 1;
  return str.substring( 0, idx);
 }

 //=====================================*/
 /** abgeschlossenen Ausdruck bergeben */
 //=====================================*/

 private String isolateTerm( String str) {
 int          cnt = 0;
 String       elm = "";
 StringBuffer trm = new StringBuffer();

  elm = isolateElm( str);
  if (elm.equals( "(")) cnt++; else return elm;

  str = str.trim();
  str = str.substring( elm.length());
  trm.append( elm);
  while( (str.length() > 0) && (cnt > 0)) {
   elm = isolateElm( str);

   if (elm.equals( "(")) cnt++;
   if (elm.equals( ")")) cnt--;

   str = str.trim();
   str = str.substring( elm.length());
   trm.append( elm);
  }
  return trm.toString();
 }

 //*======================================*/
 /** Eliminierung aller unren Operatoren */
 //*======================================*/

 private void unUnary( String str) {
 String  elm = "";
 boolean neg = false;

 String  fun = "";

  /*----------------------------*/
  /* erstes Element extrahieren */
  /*----------------------------*/

  elm = isolateElm( str);
  str = str.trim();
  str = str.substring( elm.length());

  /*---------------------------------------------------*/
  /* Schleife fr alle weiteren Elemente des Ausdrucks */
  /*---------------------------------------------------*/

  while( (str.length() > 0) && error == TEST_OK) {

   /*--------------------------------*/
   /* wenn Element eine Funktion ist */
   /*--------------------------------*/

        if (grammar.getFunctions().findFunction( elm) >= 0)
           {
            fun = elm;                          // Funktionsnamen zwischenspeichern
            elm = isolateTerm( str);            // Funktionsterm isolieren
            str = str.trim();                   // Eingang trimmen (clear white spaces)
            str = str.substring( elm.length()); // Funktionsterm am Eingang entfernen
            envir.append( elm);                 // Funktionsterm am Ausgang anhngen
            envir.append( "_" + fun);           // modifizierten Funktionsnamen anhngen
            }

   /*---------------------------------------------*/
   /* wenn Element kein Operator, dann bertragen */
   /*---------------------------------------------*/

   else if (grammar.getOperators().findOperator( elm.charAt( 0)) < 0)
            envir.append( elm);

   /*--------------------------*/
   /* Test, ob unrer Operator */
   /*--------------------------*/

   else if (!elm.equals( ")") &&
            ((str.charAt( 0) == '-') ||
             (str.charAt( 0) == '+') ||
             (str.charAt( 0) == '!')))
           {
            neg = (str.charAt( 0) == '!'); // boolsche Negation

            /*----------------------------------------------*/
            /* zwischen Addition und Negation unterscheiden */
            /*----------------------------------------------*/

            if  (!neg)
                 envir.append( elm+"(0");    // arithmetische Negation
            else envir.append( elm+"not(");  // boolsche      Negation

            /*-------------------------------------------*/
            /* Operator extrahieren und evtl. bertragen */
            /*-------------------------------------------*/

            elm = isolateElm( str);
            str = str.trim();
            str = str.substring( elm.length());

            if (!neg) envir.append( elm);

            /*---------------------------------*/
            /* Term extrahieren und bertragen */
            /*---------------------------------*/

            elm = isolateTerm( str);
            if ((grammar.getOperators().findOperator( elm.charAt( 0)) >= 0) && (elm.charAt( 0) != '('))
               {
                error = SYNTAX;
                errstr = str;
               }
            str = str.trim();
            str = str.substring( elm.length());

            envir.append( elm + ")");
           }
      else envir.append( elm);

   elm = isolateElm( str);
   str = str.trim();
   str = str.substring( elm.length());
  }
  envir.append( elm);
 }

 //*==========================*/
 /** Infix in Postfix wandeln */
 //*==========================*/

 private String toUpn( String str, boolean ok) {
 String       elm = "";
 Stack        stk = new Stack();
 StringBuffer buf = new StringBuffer();

  while( ((str.length() > 0) || (elm.length() > 0)) && ok) {
   if (elm.length() < 1)
      {
       elm = isolateElm( str);
       str = str.trim();
       str = str.substring( elm.length());
      }

        if (grammar.getOperators().findOperator( elm.charAt( 0)) < 0)
           {
            buf.append( elm+" ");
            elm = "";
           }
   else if (stk.empty())
           {
            stk.push( elm);
            elm = "";
           }
   else if (elm.equals( "("))
           {
            stk.push( elm);
            elm = "";
           }
   else if ( elm.equals( ")") && ((String)stk.peek()).equals( "("))
           {
            stk.pop();
            elm = "";
           }
   else if (grammar.getOperators().opVal( elm.charAt( 0)) > 
            grammar.getOperators().opVal( ((String)stk.peek()).charAt( 0)))
           {
            stk.push( elm);
            elm = "";
           }
   else if (grammar.getOperators().opVal( elm.charAt( 0)) <= 
            grammar.getOperators().opVal( ((String)stk.peek()).charAt( 0)))
           {
            buf.append( (String)stk.pop()+" ");
           }
  }

  /*----------------------------*/
  /* Rest des Stacks bertragen */
  /*----------------------------*/

  while( !stk.empty() && ok) buf.append( (String)stk.pop()+" ");
  return buf.toString().trim();
 }

 //*==============================================*/
 /** Spaces und berflssige Operatoren entfernen */
 //*==============================================*/

 private String compress( String str) {
 StringTokenizer tok = new StringTokenizer( str);
 int             cnt = tok.countTokens();

  /*------------------*/
  /* Spaces entfernen */
  /*------------------*/

  envir.setLength( 0);
  for( int idx = 0; idx < cnt; idx++) {
   envir.append( tok.nextToken().toString());
  }
  return envir.toString();
 }

 //*=====================================================*/
 /** Infix auf Korrektheit prfen und in Postfix wandeln */
 //*=====================================================*/

 private String postfix( String str) {
 String orgstr = str;   // ursprnglichen Ausdruck zwischenspeichern

  str = compress( str); // entfernen aller white spaces
  envir.setLength( 0);  // Environment leeren

  /*---------------------------------------------------------------*/
  /* bei fhrendem unren Operator, um eine Klammerebene erweitern */
  /*---------------------------------------------------------------*/

  if ((str.charAt( 0) == '-') || (str.charAt( 0) == '+') || (str.charAt( 0) == '!'))
     {
      envir.append( "(" + str + ")");
      str = envir.toString();
      envir.setLength( 0);
     }

  /*------------------------------------*/
  /* alle unren Operatoren eliminieren */
  /*------------------------------------*/

  while( !str.equals( envir.toString())) {
   unUnary( str);                       // einen unren Operator eliminieren
   if (!str.equals( envir.toString()))  // wenn Environment != Berarbeitungsstring
      {
       str = envir.toString();          // Berarbeitungsstring = Environment
       envir.setLength( 0);             // Environment leeren
      }
  }

  /*----------------------------------------------*/
  /* Test, ob fehlerhafter Ausdruck erkannt wurde */
  /*----------------------------------------------*/

  if (error > 0)
     {
      if ((str.charAt( 0) == '-') || (str.charAt( 0) == '+') || (str.charAt( 0) == '!'))
          errstr = errstr.substring( 0, errstr.length()-1);
      errpos = orgstr.indexOf( errstr);
      return "";
     }

  /*--------------------------------*/
  /* Umwandlung in postfix-Notation */
  /*--------------------------------*/

  return toUpn( str, error == TEST_OK);
 }

 //*********************************************************************/
 /** Im Argument vorhandenen Ausdruck in Postfix wandeln und bergeben */
 //*********************************************************************/

 public String intoPostfix( String expr) {
  return postfix( expr);
 }
 
 //***********************/
 /** Grammatik bergeben */
 //***********************/

 public Grammar getGrammar() {
  return this.grammar;
 }
 
 //***********************/
 /** Grammatik festlegen */
 //***********************/

 public void setGrammar( Grammar grammar) {
  this.grammar = grammar;
 }
 
 //***********/
 /** CREATOR */
 //***********/
 
 public Translate( Grammar grammar) {
  setGrammar( grammar);
 }
 
 public Translate() {
 }
}
